// Generated from libvo/vo_gl3_shaders.glsl

"/*\n"
" * This file is part of mplayer2.\n"
" *\n"
" * mplayer2 is free software; you can redistribute it and/or modify\n"
" * it under the terms of the GNU General Public License as published by\n"
" * the Free Software Foundation; either version 2 of the License, or\n"
" * (at your option) any later version.\n"
" *\n"
" * mplayer2 is distributed in the hope that it will be useful,\n"
" * but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
" * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
" * GNU General Public License for more details.\n"
" *\n"
" * You should have received a copy of the GNU General Public License along\n"
" * with mplayer2; if not, write to the Free Software Foundation, Inc.,\n"
" * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.\n"
" */\n"
"\n"
"// Note that this file is not directly passed as shader, but run through some\n"
"// text processing functions, and in fact contains multiple vertex and fragment\n"
"// shaders.\n"
"\n"
"// inserted at the beginning of all shaders\n"
"#!section prelude\n"
"#!section vertex_all\n"
"uniform mat3 transform;\n"
"uniform sampler3D lut_3d;\n"
"\n"
"in vec2 vertex_position;\n"
"in vec4 vertex_color;\n"
"out vec4 color;\n"
"in vec2 vertex_texcoord;\n"
"out vec2 texcoord;\n"
"\n"
"void main() {\n"
"    vec3 position = vec3(vertex_position, 1);\n"
"#ifndef FIXED_SCALE\n"
"    position = transform * position;\n"
"#endif\n"
"    gl_Position = vec4(position, 1);\n"
"    color = vertex_color;\n"
"#ifdef USE_3DLUT\n"
"    color = vec4(texture(lut_3d, color.rgb).rgb, color.a);\n"
"#endif\n"
"    texcoord = vertex_texcoord;\n"
"}\n"
"\n"
"#!section frag_eosd\n"
"uniform sampler2D texture1;\n"
"\n"
"in vec2 texcoord;\n"
"in vec4 color;\n"
"out vec4 out_color;\n"
"\n"
"void main() {\n"
"    out_color = vec4(color.rgb, color.a * texture(texture1, texcoord).r);\n"
"}\n"
"\n"
"#!section frag_osd\n"
"uniform sampler2D texture1;\n"
"\n"
"in vec2 texcoord;\n"
"in vec4 color;\n"
"out vec4 out_color;\n"
"\n"
"void main() {\n"
"    out_color = texture(texture1, texcoord).rrrg * color;\n"
"}\n"
"\n"
"#!section frag_video\n"
"uniform sampler2D texture1;\n"
"uniform sampler2D texture2;\n"
"uniform sampler2D texture3;\n"
"uniform sampler1D lut_c_1d;\n"
"uniform sampler1D lut_l_1d;\n"
"uniform sampler2D lut_c_2d;\n"
"uniform sampler2D lut_l_2d;\n"
"uniform sampler3D lut_3d;\n"
"uniform sampler2D dither;\n"
"uniform mat4x3 colormatrix;\n"
"uniform vec3 inv_gamma;\n"
"uniform float conv_gamma;\n"
"uniform float dither_quantization;\n"
"uniform float dither_multiply;\n"
"uniform float filter_param1;\n"
"\n"
"in vec2 texcoord;\n"
"out vec4 out_color;\n"
"\n"
"vec4 sample_bilinear(sampler2D tex, vec2 texcoord) {\n"
"    return texture(tex, texcoord);\n"
"}\n"
"\n"
"// Explanation how bicubic scaling with only 4 texel fetches is done:\n"
"//   http://www.mate.tue.nl/mate/pdfs/10318.pdf\n"
"//   'Efficient GPU-Based Texture Interpolation using Uniform B-Splines'\n"
"// Explanation why this algorithm normally always blurs, even with unit scaling:\n"
"//   http://bigwww.epfl.ch/preprints/ruijters1001p.pdf\n"
"//   'GPU Prefilter for Accurate Cubic B-spline Interpolation'\n"
"vec4 calcweights(float s) {\n"
"    vec4 t = vec4(-0.5, 0.1666, 0.3333, -0.3333) * s + vec4(1, 0, -0.5, 0.5);\n"
"    t = t * s + vec4(0, 0, -0.5, 0.5);\n"
"    t = t * s + vec4(-0.6666, 0, 0.8333, 0.1666);\n"
"    vec2 a = vec2(1 / t.z, 1 / t.w);\n"
"    t.xy = t.xy * a + vec2(1, 1);\n"
"    t.x = t.x + s;\n"
"    t.y = t.y - s;\n"
"    return t;\n"
"}\n"
"\n"
"vec4 sample_bicubic_fast(sampler2D tex, vec2 texcoord) {\n"
"    vec2 texsize = textureSize(tex, 0);\n"
"    vec2 pt = 1 / texsize;\n"
"    vec2 fcoord = fract(texcoord * texsize + vec2(0.5, 0.5));\n"
"    vec4 parmx = calcweights(fcoord.x);\n"
"    vec4 parmy = calcweights(fcoord.y);\n"
"    vec4 cdelta;\n"
"    cdelta.xz = parmx.rg * vec2(-pt.x, pt.x);\n"
"    cdelta.yw = parmy.rg * vec2(-pt.y, pt.y);\n"
"    // first y-interpolation\n"
"    vec4 ar = texture(tex, texcoord + cdelta.xy);\n"
"    vec4 ag = texture(tex, texcoord + cdelta.xw);\n"
"    vec4 ab = mix(ag, ar, parmy.b);\n"
"    // second y-interpolation\n"
"    vec4 br = texture(tex, texcoord + cdelta.zy);\n"
"    vec4 bg = texture(tex, texcoord + cdelta.zw);\n"
"    vec4 aa = mix(bg, br, parmy.b);\n"
"    // x-interpolation\n"
"    return mix(aa, ab, parmx.b);\n"
"}\n"
"\n"
"float[2] weights2(sampler1D lookup, float f) {\n"
"    vec4 c = texture(lookup, f);\n"
"    return float[2](c.r, c.g);\n"
"}\n"
"\n"
"float[4] weights4(sampler1D lookup, float f) {\n"
"    vec4 c = texture(lookup, f);\n"
"    return float[4](c.r, c.g, c.b, c.a);\n"
"}\n"
"\n"
"float[6] weights6(sampler2D lookup, float f) {\n"
"    vec4 c1 = texture(lookup, vec2(0.25, f));\n"
"    vec4 c2 = texture(lookup, vec2(0.75, f));\n"
"    return float[6](c1.r, c1.g, c1.b, c2.r, c2.g, c2.b);\n"
"}\n"
"\n"
"float[8] weights8(sampler2D lookup, float f) {\n"
"    vec4 c1 = texture(lookup, vec2(0.25, f));\n"
"    vec4 c2 = texture(lookup, vec2(0.75, f));\n"
"    return float[8](c1.r, c1.g, c1.b, c1.a, c2.r, c2.g, c2.b, c2.a);\n"
"}\n"
"\n"
"float[12] weights12(sampler2D lookup, float f) {\n"
"    vec4 c1 = texture(lookup, vec2(1.0/6.0, f));\n"
"    vec4 c2 = texture(lookup, vec2(0.5, f));\n"
"    vec4 c3 = texture(lookup, vec2(5.0/6.0, f));\n"
"    return float[12](c1.r, c1.g, c1.b, c1.a,\n"
"                     c2.r, c2.g, c2.b, c2.a,\n"
"                     c3.r, c3.g, c3.b, c3.a);\n"
"}\n"
"\n"
"float[16] weights16(sampler2D lookup, float f) {\n"
"    vec4 c1 = texture(lookup, vec2(0.125, f));\n"
"    vec4 c2 = texture(lookup, vec2(0.375, f));\n"
"    vec4 c3 = texture(lookup, vec2(0.625, f));\n"
"    vec4 c4 = texture(lookup, vec2(0.875, f));\n"
"    return float[16](c1.r, c1.g, c1.b, c1.a, c2.r, c2.g, c2.b, c2.a,\n"
"                     c3.r, c3.g, c3.b, c3.a, c4.r, c4.g, c4.b, c4.a);\n"
"}\n"
"\n"
"#define CONVOLUTION_SEP_N(NAME, N)                                           \\\n"
"    vec4 NAME(sampler2D tex, vec2 texcoord, vec2 pt, float weights[N]) {     \\\n"
"        vec4 res = vec4(0);                                                  \\\n"
"        for (int n = 0; n < N; n++) {                                        \\\n"
"            res += weights[n] * texture(tex, texcoord + pt * n);             \\\n"
"        }                                                                    \\\n"
"        return res;                                                          \\\n"
"    }\n"
"\n"
"CONVOLUTION_SEP_N(convolution_sep2, 2)\n"
"CONVOLUTION_SEP_N(convolution_sep4, 4)\n"
"CONVOLUTION_SEP_N(convolution_sep6, 6)\n"
"CONVOLUTION_SEP_N(convolution_sep8, 8)\n"
"CONVOLUTION_SEP_N(convolution_sep12, 12)\n"
"CONVOLUTION_SEP_N(convolution_sep16, 16)\n"
"\n"
"// The dir parameter is (0, 1) or (1, 0), and we expect the shader compiler to\n"
"// remove all the redundant multiplications and additions.\n"
"#define SAMPLE_CONVOLUTION_SEP_N(NAME, N, SAMPLERT, CONV_FUNC, WEIGHTS_FUNC)\\\n"
"    vec4 NAME(vec2 dir, SAMPLERT lookup, sampler2D tex, vec2 texcoord) {    \\\n"
"        vec2 texsize = textureSize(tex, 0);                                 \\\n"
"        vec2 pt = (1 / texsize) * dir;                                      \\\n"
"        float fcoord = dot(fract(texcoord * texsize - 0.5), dir);           \\\n"
"        vec2 base = texcoord - fcoord * pt;                                 \\\n"
"        return CONV_FUNC(tex, base - pt * (N / 2 - 1), pt,                  \\\n"
"                         WEIGHTS_FUNC(lookup, fcoord));                     \\\n"
"    }\n"
"\n"
"SAMPLE_CONVOLUTION_SEP_N(sample_convolution_sep2, 2, sampler1D, convolution_sep2, weights2)\n"
"SAMPLE_CONVOLUTION_SEP_N(sample_convolution_sep4, 4, sampler1D, convolution_sep4, weights4)\n"
"SAMPLE_CONVOLUTION_SEP_N(sample_convolution_sep6, 6, sampler2D, convolution_sep6, weights6)\n"
"SAMPLE_CONVOLUTION_SEP_N(sample_convolution_sep8, 8, sampler2D, convolution_sep8, weights8)\n"
"SAMPLE_CONVOLUTION_SEP_N(sample_convolution_sep12, 12, sampler2D, convolution_sep12, weights12)\n"
"SAMPLE_CONVOLUTION_SEP_N(sample_convolution_sep16, 16, sampler2D, convolution_sep16, weights16)\n"
"\n"
"\n"
"#define CONVOLUTION_N(NAME, N)                                               \\\n"
"    vec4 NAME(sampler2D tex, vec2 texcoord, vec2 pt, float taps_x[N],        \\\n"
"              float taps_y[N]) {                                             \\\n"
"        vec4 res = vec4(0);                                                  \\\n"
"        for (int y = 0; y < N; y++) {                                        \\\n"
"            vec4 line = vec4(0);                                             \\\n"
"            for (int x = 0; x < N; x++)                                      \\\n"
"                line += taps_x[x] * texture(tex, texcoord + pt * vec2(x, y));\\\n"
"            res += taps_y[y] * line;                                         \\\n"
"        }                                                                    \\\n"
"        return res;                                                          \\\n"
"    }\n"
"\n"
"CONVOLUTION_N(convolution2, 2)\n"
"CONVOLUTION_N(convolution4, 4)\n"
"CONVOLUTION_N(convolution6, 6)\n"
"CONVOLUTION_N(convolution8, 8)\n"
"CONVOLUTION_N(convolution12, 12)\n"
"CONVOLUTION_N(convolution16, 16)\n"
"\n"
"#define SAMPLE_CONVOLUTION_N(NAME, N, SAMPLERT, CONV_FUNC, WEIGHTS_FUNC)    \\\n"
"    vec4 NAME(SAMPLERT lookup, sampler2D tex, vec2 texcoord) {              \\\n"
"        vec2 texsize = textureSize(tex, 0);                                 \\\n"
"        vec2 pt = 1 / texsize;                                              \\\n"
"        vec2 fcoord = fract(texcoord * texsize - 0.5);                      \\\n"
"        vec2 base = texcoord - fcoord * pt;                                 \\\n"
"        return CONV_FUNC(tex, base - pt * (N / 2 - 1), pt,                  \\\n"
"                         WEIGHTS_FUNC(lookup, fcoord.x),                    \\\n"
"                         WEIGHTS_FUNC(lookup, fcoord.y));                   \\\n"
"    }\n"
"\n"
"SAMPLE_CONVOLUTION_N(sample_convolution2, 2, sampler1D, convolution2, weights2)\n"
"SAMPLE_CONVOLUTION_N(sample_convolution4, 4, sampler1D, convolution4, weights4)\n"
"SAMPLE_CONVOLUTION_N(sample_convolution6, 6, sampler2D, convolution6, weights6)\n"
"SAMPLE_CONVOLUTION_N(sample_convolution8, 8, sampler2D, convolution8, weights8)\n"
"SAMPLE_CONVOLUTION_N(sample_convolution12, 12, sampler2D, convolution12, weights12)\n"
"SAMPLE_CONVOLUTION_N(sample_convolution16, 16, sampler2D, convolution16, weights16)\n"
"\n"
"\n"
"// Unsharp masking\n"
"vec4 sample_sharpen3(sampler2D tex, vec2 texcoord) {\n"
"    vec2 texsize = textureSize(tex, 0);\n"
"    vec2 pt = 1 / texsize;\n"
"    vec2 st = pt * 0.5;\n"
"    vec4 p = texture(tex, texcoord);\n"
"    vec4 sum = texture(tex, texcoord + st * vec2(+1, +1))\n"
"             + texture(tex, texcoord + st * vec2(+1, -1))\n"
"             + texture(tex, texcoord + st * vec2(-1, +1))\n"
"             + texture(tex, texcoord + st * vec2(-1, -1));\n"
"    return p + (p - 0.25 * sum) * filter_param1;\n"
"}\n"
"\n"
"vec4 sample_sharpen5(sampler2D tex, vec2 texcoord) {\n"
"    vec2 texsize = textureSize(tex, 0);\n"
"    vec2 pt = 1 / texsize;\n"
"    vec2 st1 = pt * 1.2;\n"
"    vec4 p = texture(tex, texcoord);\n"
"    vec4 sum1 = texture(tex, texcoord + st1 * vec2(+1, +1))\n"
"              + texture(tex, texcoord + st1 * vec2(+1, -1))\n"
"              + texture(tex, texcoord + st1 * vec2(-1, +1))\n"
"              + texture(tex, texcoord + st1 * vec2(-1, -1));\n"
"    vec2 st2 = pt * 1.5;\n"
"    vec4 sum2 = texture(tex, texcoord + st2 * vec2(+1,  0))\n"
"              + texture(tex, texcoord + st2 * vec2( 0, +1))\n"
"              + texture(tex, texcoord + st2 * vec2(-1,  0))\n"
"              + texture(tex, texcoord + st2 * vec2( 0, -1));\n"
"    vec4 t = p * 0.859375 + sum2 * -0.1171875 + sum1 * -0.09765625;\n"
"    return p + t * filter_param1;\n"
"}\n"
"\n"
"void main() {\n"
"#ifdef USE_PLANAR\n"
"    vec3 color = vec3(SAMPLE_L(texture1, texcoord).r,\n"
"                      SAMPLE_C(texture2, texcoord).r,\n"
"                      SAMPLE_C(texture3, texcoord).r);\n"
"#else\n"
"    vec3 color = SAMPLE_L(texture1, texcoord).rgb;\n"
"#endif\n"
"#ifdef USE_GBRP\n"
"    color.gbr = color;\n"
"#endif\n"
"#ifdef USE_YGRAY\n"
"    // NOTE: actually slightly wrong for 16 bit input video, and completely\n"
"    //       wrong for 9/10 bit input\n"
"    color.gb = vec2(128.0/255.0);\n"
"#endif\n"
"#ifdef USE_COLORMATRIX\n"
"    color = mat3(colormatrix) * color + colormatrix[3];\n"
"#endif\n"
"#ifdef USE_LINEAR_CONV\n"
"    color = pow(color, vec3(2.2));\n"
"#endif\n"
"#ifdef USE_LINEAR_CONV_INV\n"
"    // Convert from linear RGB to gamma RGB before putting it through the 3D-LUT\n"
"    // in the final stage.\n"
"    color = pow(color, vec3(1.0/2.2));\n"
"#endif\n"
"#ifdef USE_GAMMA_POW\n"
"    color = pow(color, inv_gamma);\n"
"#endif\n"
"#ifdef USE_3DLUT\n"
"    color = texture(lut_3d, color).rgb;\n"
"#endif\n"
"#ifdef USE_DITHER\n"
"    float dither = texture(dither, gl_FragCoord.xy / textureSize(dither, 0)).r;\n"
"    color = floor(color * dither_multiply + dither ) / dither_quantization;\n"
"#endif\n"
"    out_color = vec4(color, 1);\n"
"}\n"
